//---------------------------------------------------------------------------
// File:          mocam_Autofocus.h
// Description:   C99 and C++ API for MoCam autofocus library
// Copyright (C)2010 Mobiclip, Inc.
//---------------------------------------------------------------------------

#ifndef mocam_Autofocus_h
#define mocam_Autofocus_h

//---------------------------------------------------------------------------

#include "mocam_Defs.h"

//---------------------------------------------------------------------------
// C99 API interface.
//---------------------------------------------------------------------------

#ifdef __cplusplus
	extern "C" {
#endif

//---------------------------------------------------------------------------
// Internal library data.
//---------------------------------------------------------------------------

typedef struct mwmomocamautofocusHandle mwmomocamautofocusHandle;

//---------------------------------------------------------------------------
// C99 API
//---------------------------------------------------------------------------

u32		mwmomocamautofocusHandleInitialize	(mwmomocamautofocusHandle** ppHandle,
											 mwmomocamMalloc malloc, 
											 mwmomocamFree free);

void	mwmomocamautofocusHandleFinalize	(mwmomocamautofocusHandle* pHandle);

u32		mwmomocamautofocusHandleCalculate	(mwmomocamautofocusHandle* pHandle,
											 const u8* src0,const u8* src1,
											 u32 width,u32 height,u32 stride,
											 s32 maxParallax,
											 bool formatPlanarY);

u32		mwmomocamautofocusHandleGet		(mwmomocamautofocusHandle* pHandle,f32 dampingFactor,f32* focus);

u32		mwmomocamautofocusHandleSet		(mwmomocamautofocusHandle* pHandle,f32 focus);

u32		mwmomocamautofocusHandleSetCallback	(mwmomocamautofocusHandle* pHandle,void (*callback)(void*),void* param);

//---------------------------------------------------------------------------

#ifdef __cplusplus
}
#endif

//---------------------------------------------------------------------------
// C++ API interface
//---------------------------------------------------------------------------

#ifdef __cplusplus

namespace mw { 
namespace mo {
namespace mocam {
namespace autofocus {

//---------------------------------------------------------------------------
// C++ wrapper for C99 functions
//---------------------------------------------------------------------------

class Handle
{
protected:
	mwmomocamautofocusHandle* m_Handle;

public:
	inline Handle():m_Handle(0) {}

	inline ~Handle()
	{
		Finalize();
	}

	inline ErrorCodes Initialize(Malloc malloc,Free free)
	{
		if (!m_Handle)
			return (ErrorCodes)mwmomocamautofocusHandleInitialize(&m_Handle,malloc,free);
		return OK;
	}

	inline void Finalize(void)
	{
		if (m_Handle)
		{
			mwmomocamautofocusHandleFinalize(m_Handle);
			m_Handle=0;
		}
	}	

	inline ErrorCodes Calculate(const u8* src0,const u8* src1,
		 					    u32 width,u32 height,u32 stride,
							    s32 maxParallax,
							    bool formatPlanarY=false)
	{
		if (!m_Handle)
			return NOT_INITIALIZED;
		return (ErrorCodes)mwmomocamautofocusHandleCalculate(m_Handle,src0,src1,width,height,stride,maxParallax,formatPlanarY);
	}

	inline ErrorCodes Get(f32 dampingFactor,f32& focus)
	{
		if (!m_Handle)
			return NOT_INITIALIZED;
		return (ErrorCodes)mwmomocamautofocusHandleGet(m_Handle,dampingFactor,&focus);
	}

	inline ErrorCodes Set(f32 focus)
	{
		if (!m_Handle)
			return NOT_INITIALIZED;
		return (ErrorCodes)mwmomocamautofocusHandleSet(m_Handle,focus);
	}
	
	inline ErrorCodes SetCallback(void (*callback)(void*),void* param)
	{
		if (!m_Handle)
			return NOT_INITIALIZED;
		return (ErrorCodes)mwmomocamautofocusHandleSetCallback(m_Handle,callback,param);
	}
};

/*---------------------------------------------------------------------------*/

} // end of namespace autofocus
} // end of namespace mocam
} // end of namespace mo
} // end of namespace mw

#endif // ifdef __cplusplus

/*---------------------------------------------------------------------------*/

#endif // mocam_Autofocus_h

/*---------------------------------------------------------------------------*/
