//---------------------------------------------------------------------------
// File:          mocam_Defs.h
// Description:   C99 and C++ API for MoCam library
// Copyright (C)2010-2011 Mobiclip, Inc.
//---------------------------------------------------------------------------

#ifndef mocam_defs_h
#define mocam_defs_h

//---------------------------------------------------------------------------

#include <nn.h>

//---------------------------------------------------------------------------
// C99 API interface.
//---------------------------------------------------------------------------

#ifdef __cplusplus
	extern "C" {
#endif

//---------------------------------------------------------------------------
// Possible return values (error codes).
//---------------------------------------------------------------------------

#define MWMOMOCAM_ERRORCODES_OK							0
#define MWMOMOCAM_ERRORCODES_NOT_ENOUGH_MEMORY			1
#define MWMOMOCAM_ERRORCODES_NOT_INITIALIZED			2
#define MWMOMOCAM_ERRORCODES_INTERNAL_ERROR				128

//---------------------------------------------------------------------------
// Malloc and free function prototypes used by the calibration.
//---------------------------------------------------------------------------

typedef void*	(*mwmomocamMalloc)				(size_t size);
typedef void	(*mwmomocamFree)				(void* ptr);

//---------------------------------------------------------------------------
// Calculate relative parallax at a given distance (in meters).
//---------------------------------------------------------------------------

f32		mwmomocamDistanceToParallax			(u32 width,f32 angleOfView,f32 distance);

//---------------------------------------------------------------------------

#ifdef __cplusplus
}
#endif

//---------------------------------------------------------------------------
// C++ API interface
//---------------------------------------------------------------------------

#ifdef __cplusplus

namespace mw { 
namespace mo {
namespace mocam {

//---------------------------------------------------------------------------
// Error codes
//---------------------------------------------------------------------------

enum ErrorCodes
{
	OK					= MWMOMOCAM_ERRORCODES_OK,	
	NOT_ENOUGH_MEMORY	= MWMOMOCAM_ERRORCODES_NOT_ENOUGH_MEMORY,
	NOT_INITIALIZED		= MWMOMOCAM_ERRORCODES_NOT_INITIALIZED,
	INTERNAL_ERROR		= MWMOMOCAM_ERRORCODES_INTERNAL_ERROR
};

//---------------------------------------------------------------------------
// C++ wrapper for C99 functions
//---------------------------------------------------------------------------

typedef mwmomocamMalloc		Malloc;
typedef mwmomocamFree		Free;

//---------------------------------------------------------------------------

inline f32 DistanceToParallax(u32 width,f32 angleOfView,f32 distance)
{
	return mwmomocamDistanceToParallax(width,angleOfView,distance);
}

/*---------------------------------------------------------------------------*/

} // end of namespace mocam
} // end of namespace mo
} // end of namespace mw

#endif // ifdef __cplusplus

/*---------------------------------------------------------------------------*/

#endif // mocam_defs_h

/*---------------------------------------------------------------------------*/
