/*---------------------------------------------------------------------------*
 File:        helper_Camera.h
 Description: Helper file that manages camera capture

 Copyright (C)2011 Mobiclip, Inc.
 *---------------------------------------------------------------------------*/

#ifndef helper_Camera_h
#define helper_Camera_h

/*---------------------------------------------------------------------------*/

#include <nn/types.h>
#include <nn/fnd.h>
#include <nn/camera/CTR/camera_Api.h>

/*---------------------------------------------------------------------------*
  C++ API interface
 *---------------------------------------------------------------------------*/

#ifdef __cplusplus

/*---------------------------------------------------------------------------*/

namespace mw {
namespace mo {
namespace helper {

/*---------------------------------------------------------------------------*/

#define NB_CAMERA_BUFFER    3

/*---------------------------------------------------------------------------*/

class Camera
{
    private:

    nn::fnd::IAllocator* m_Allocator;
    int                  m_NbVideoImage;
    int                  m_Width;
    int                  m_Height;
    int                  m_SelectionPosX;
    int                  m_SelectionPosY;
    int                  m_SelectionWidth;
    int                  m_SelectionHeight;
    u8*                  m_Yuv422VideoImageData[2][NB_CAMERA_BUFFER];
    nn::os::Event        m_ErrorEvent[2];
    nn::os::Event        m_ReceiveEvent[2];
    nn::os::Event        m_VSyncEvent[2];
    nn::os::Event        m_EndCaptureEvent;    
    volatile int         m_CapturedFrameIndex[2];
    volatile int         m_RetrievedFrameIndex[2];
    s16                  m_TransferUnit;
    s16                  m_Padding;
    size_t               m_CameraOutputSize;
    
    nn::os::Thread       m_CaptureThread;
    nn::os::Event*       m_EventArray[5];
    nn::os::LightEvent   m_ClientEvent;
    int                  m_NbEvents;
    bool                 m_Capturing;

    int:                 24; // padding added to avoid compiler warning

    static void CaptureThreadFunc (Camera* camera);

    public:

    Camera()
    {
        m_Allocator = NULL;
        m_NbVideoImage = 0;
        m_Yuv422VideoImageData[0][0] = NULL;
        m_Yuv422VideoImageData[0][1] = NULL;
        m_Yuv422VideoImageData[1][0] = NULL;
        m_Yuv422VideoImageData[1][1] = NULL;
        m_Capturing = false;
    }

    bool Initialize (nn::fnd::IAllocator* pAllocator,
                     nn::camera::CTR::CameraSelect cameraSelect, 
                     nn::camera::CTR::Size size, 
                     nn::camera::CTR::FrameRate framerate);
    bool Initialize (nn::fnd::IAllocator* pAllocator,
                     nn::camera::CTR::CameraSelect cameraSelect,
                     nn::camera::CTR::FrameRate framerate,
                     int width, int height,
                     int selectionPosX = -1, int selectionPosY = -1,
                     int selectionWidth = 0, int selectionHeight = 0);
    void Finalize   (void);

    bool StartCapture      (void);
    bool StopCapture       (void);
    bool IsImageAvailable  (void);
    void WaitNextImage     (void);
    void PopImage          (void);
    int  GetNbImageChannel (void){return m_NbVideoImage;}
    u8*  GetImageChannel   (int channelIndex){return m_Yuv422VideoImageData[m_NbVideoImage-1-channelIndex][m_RetrievedFrameIndex[m_NbVideoImage-1-channelIndex]%NB_CAMERA_BUFFER];}
    void ChangeResolution  (nn::camera::CameraSelect cameraSelect, int width, int height);
};

/*---------------------------------------------------------------------------*/

} // end of namespace helper
} // end of namespace mo
} // end of namespace mw

/*---------------------------------------------------------------------------*/

#endif // ifdef __cplusplus

/*---------------------------------------------------------------------------*/

#endif // ifndef helper_Camera_h

/*---------------------------------------------------------------------------*/
