/*---------------------------------------------------------------------------*
 File:        helper_DisplayDevice.h
 Description: Helper file that manages CTR display presentation functions in 
              mono(2D) and stereo(3D) mode.

 Copyright (C)2011 Mobiclip, Inc.
 *---------------------------------------------------------------------------*/

#ifndef helper_DisplayDevice_h
#define helper_DisplayDevice_h

/*---------------------------------------------------------------------------*/

#include <nn/types.h>

/*---------------------------------------------------------------------------*
  C++ API interface
 *---------------------------------------------------------------------------*/

#ifdef __cplusplus

/*---------------------------------------------------------------------------*/

namespace mw {
namespace mo {
namespace helper {

/*---------------------------------------------------------------------------*
  
 *---------------------------------------------------------------------------*/

namespace displaydevice
{
    const u32 ENABLE_3D                  = 1;
    const u32 ENABLE_VRAM_DISPLAY_BUFFER = 2;

    bool  Initialize     (nn::fnd::IAllocator* pAllocator, u32 displayOptions = ENABLE_3D);
    bool  Finalize       (void);
    bool  Enable3d       (bool enable3D);
    bool  Is3dEnabled    (void);
    void* GetBackBuffer  (s32 display);
    void* GetFrontBuffer (s32 display);
    void  Flip           (s32 display);

    void BeginDraw  (s32 display);
    void EndDraw    (void);
    void Clear      (const f32 red, const f32 green, const f32 blue, const f32 alpha);
    void CommitDraw (void);
} // end of namespace displaydevice

/*---------------------------------------------------------------------------*/

} // end of namespace helper
} // end of namespace mo
} // end of namespace mw

/*---------------------------------------------------------------------------*/

#endif // ifdef __cplusplus

/*---------------------------------------------------------------------------*/

#endif // ifndef helper_DisplayDevice_h

/*---------------------------------------------------------------------------*/

