/*---------------------------------------------------------------------------*
 File:          helper_RgbImage.h
 Description:   Helper file that manages 2D drawing of RGB textured images.

 Copyright (C)2011 Mobiclip, Inc.
 *---------------------------------------------------------------------------*/

#ifndef helper_RgbImage2d_h
#define helper_RgbImage2d_h

/*---------------------------------------------------------------------------*/

#include <nn/types.h>
#include <nn/math.h>
#include <nn/gx.h>

/*---------------------------------------------------------------------------*
  C++ API interface
 *---------------------------------------------------------------------------*/

#ifdef __cplusplus

/*---------------------------------------------------------------------------*/

namespace mw {
namespace mo {
namespace helper {

/*---------------------------------------------------------------------------*/

class RgbImage
{
    private:
    
    GLuint          m_TextureId;
    GLuint          m_VertexPositionID;
    GLuint          m_VertexTexCoordID;
    GLuint          m_ProjectionMatrixID;
    GLuint          m_ModelViewMatrixID;
    int             m_TextureWidth;
    int             m_TextureHeight;
    GLenum          m_Format;
    GLenum          m_Type;
    void*           m_Pixels;
    nn::math::VEC3  m_VertexPosition[4];
    nn::math::VEC2  m_VertexTexCoord[4];
    nn::math::MTX44 m_OrthoProjection;
    nn::math::MTX44 m_ModelView;
    GLfloat         m_Transparency;
    bool            m_TextureAlpha;
    bool            m_TransparencyAlpha;
    int:16;

    void InternalCreateTexture (GLenum target);
    void InternalDeleteTexture (void);

    public:

    RgbImage() : m_Pixels(NULL){}

    void Initialize (const int textureWidth, const int textureHeight,
                     const int imagePosX, const int imagePosY,
                     const int imageWidth, const int imageHeight,
                     const GLenum format, const GLenum type, void* pixels,GLenum target=0);
    void Finalize   (void);

    void SetImagePosition  (float centerX, float centerY, float width, float height);
    void SetImageSelection (const int imagePosX, const int imagePosY,
                            const int imageWidth, const int imageHeight);
    void SetImageRotation  (float clockwiseDegreeRotationZ);

    void SetModelViewMatrix  (nn::math::MTX44& modelView);
    void SetProjectionMatrix (nn::math::MTX44& projection);
    void SetTransparency     (float transparency);
    void EnableAlpha         (void) { m_TransparencyAlpha=true; }
    void DisableAlpha        (void) { m_TransparencyAlpha=false; }
    void EnableTextureAlpha  (void) { m_TextureAlpha=true; }
    void DisableTextureAlpha (void) { m_TextureAlpha=false; }

    void Draw (void);
};

/*---------------------------------------------------------------------------*/

} // end of namespace helper
} // end of namespace mo
} // end of namespace mw

/*---------------------------------------------------------------------------*/

#endif  // ifdef __cplusplus

/*---------------------------------------------------------------------------*/

#endif  // ifndef helper_RgbImage_h

/*---------------------------------------------------------------------------*/

